import type { CommonEditor, MarkupString } from '../common';
import type { ExtensionSpec } from './types/extension';
import type { ActionStorage } from './types/actions';
declare type OnChange = (editor: YfmEditor) => void;
export declare type YfmEditorOptions = {
    domElem?: Element;
    /** yfm markup */
    initialContent?: string;
    extensions?: ExtensionSpec[];
    allowHTML?: boolean;
    linkify?: boolean;
    /** markdown-it-attrs options */
    attrs?: {
        leftDelimiter?: string;
        rightDelimiter?: string;
    };
    /** Call on any state change (move cursor, change selection, etc...) */
    onChange?: OnChange;
    /** Call only if document change */
    onDocChange?: OnChange;
};
export declare class YfmEditor implements CommonEditor, ActionStorage {
    #private;
    get dom(): HTMLElement;
    get actions(): YfmEditor.Actions;
    constructor({ domElem, initialContent, extensions, attrs: attrsOpts, allowHTML, linkify, onChange, onDocChange, }: YfmEditorOptions);
    action<T extends keyof YfmEditor.Actions>(actionName: T): YfmEditor.Actions[T];
    focus(): void;
    hasFocus(): boolean;
    getValue(): MarkupString;
    isEmpty(): boolean;
    clear(): void;
    replace(newMarkup: MarkupString): void;
    prepend(markup: MarkupString): void;
    append(markup: MarkupString): void;
    moveCursor(position: 'start' | 'end'): void;
    destroy(): void;
    private tryOnChange;
}
declare global {
    namespace YfmEditor {
        interface Actions {
        }
    }
}
export {};
