import type MarkdownIt from 'markdown-it';
import type { Plugin } from 'prosemirror-state';
import { inputRules } from 'prosemirror-inputrules';
import type { Keymap } from './types/keymap';
import type { ActionSpec } from './types/actions';
import type { Extension, ExtensionWithOptions, ExtensionDeps, ExtensionSpec, YEMarkSpec, YENodeSpec } from './types/extension';
declare type InputRulesConfig = Parameters<typeof inputRules>[0];
declare type ConfigureMdCallback = (md: MarkdownIt) => MarkdownIt;
declare type AddPmNodeCallback = () => YENodeSpec;
declare type AddPmMarkCallback = () => YEMarkSpec;
declare type AddPmPluginCallback = (deps: ExtensionDeps) => Plugin | Plugin[];
declare type AddPmKeymapCallback = (deps: ExtensionDeps) => Keymap;
declare type AddPmInputRulesCallback = (deps: ExtensionDeps) => InputRulesConfig;
declare type AddActionCallback = (deps: ExtensionDeps) => ActionSpec;
declare enum PluginPriority {
    Highest = 1000000,
    VeryHigh = 100000,
    High = 10000,
    Medium = 1000,
    Low = 100,
    VeryLow = 10,
    Lowest = 0
}
export declare class ExtensionBuilder {
    #private;
    static readonly PluginPriority: typeof PluginPriority;
    readonly PluginPriority: typeof PluginPriority;
    use(extension: Extension): this;
    use<T>(extension: ExtensionWithOptions<T>, options: T): this;
    configureMd(cb: ConfigureMdCallback): this;
    addNode(name: string, cb: AddPmNodeCallback): this;
    addMark(name: string, cb: AddPmMarkCallback): this;
    addPlugin(cb: AddPmPluginCallback, priority?: PluginPriority): this;
    addKeymap(cb: AddPmKeymapCallback, priority?: PluginPriority): this;
    addInputRules(cb: AddPmInputRulesCallback, priority?: PluginPriority): this;
    addAction(name: string, cb: AddActionCallback): this;
    build(): ExtensionSpec;
}
export {};
