"use strict";
var _ExtensionBuilder_confMdCbs, _ExtensionBuilder_nodeSpecs, _ExtensionBuilder_markSpecs, _ExtensionBuilder_plugins, _ExtensionBuilder_actions;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionBuilder = void 0;
const tslib_1 = require("tslib");
const orderedmap_1 = tslib_1.__importDefault(require("orderedmap"));
const prosemirror_keymap_1 = require("prosemirror-keymap");
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
var PluginPriority;
(function (PluginPriority) {
    PluginPriority[PluginPriority["Highest"] = 1000000] = "Highest";
    PluginPriority[PluginPriority["VeryHigh"] = 100000] = "VeryHigh";
    PluginPriority[PluginPriority["High"] = 10000] = "High";
    PluginPriority[PluginPriority["Medium"] = 1000] = "Medium";
    PluginPriority[PluginPriority["Low"] = 100] = "Low";
    PluginPriority[PluginPriority["VeryLow"] = 10] = "VeryLow";
    PluginPriority[PluginPriority["Lowest"] = 0] = "Lowest";
})(PluginPriority || (PluginPriority = {}));
const DEFAULT_PRIORITY = PluginPriority.Medium;
class ExtensionBuilder {
    constructor() {
        this.PluginPriority = ExtensionBuilder.PluginPriority;
        _ExtensionBuilder_confMdCbs.set(this, []);
        _ExtensionBuilder_nodeSpecs.set(this, []);
        _ExtensionBuilder_markSpecs.set(this, []);
        _ExtensionBuilder_plugins.set(this, []);
        _ExtensionBuilder_actions.set(this, []);
    }
    use(extension, ...params) {
        extension(this, ...params);
        return this;
    }
    configureMd(cb) {
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_confMdCbs, "f").push(cb);
        return this;
    }
    addNode(name, cb) {
        if (tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_nodeSpecs, "f").some(([specName]) => specName === name)) {
            throw new Error(`ProseMirror node with this name "${name}" already exist`);
        }
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_nodeSpecs, "f").push([name, cb]);
        return this;
    }
    addMark(name, cb) {
        if (tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_markSpecs, "f").some(([specName]) => specName === name)) {
            throw new Error(`ProseMirror mark with this name "${name}" already exist`);
        }
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_markSpecs, "f").push([name, cb]);
        return this;
    }
    addPlugin(cb, priority = DEFAULT_PRIORITY) {
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_plugins, "f").push({ cb, priority });
        return this;
    }
    addKeymap(cb, priority = DEFAULT_PRIORITY) {
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_plugins, "f").push({ cb: (...args) => (0, prosemirror_keymap_1.keymap)(cb(...args)), priority });
        return this;
    }
    addInputRules(cb, priority = DEFAULT_PRIORITY) {
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_plugins, "f").push({ cb: (...args) => (0, prosemirror_inputrules_1.inputRules)(cb(...args)), priority });
        return this;
    }
    addAction(name, cb) {
        if (tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_actions, "f").some(([actionName]) => actionName === name)) {
            throw new Error(`[YFM Editor] action with this name "${name}" already exist`);
        }
        tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_actions, "f").push([name, cb]);
        return this;
    }
    build() {
        const confMd = tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_confMdCbs, "f").slice();
        const nodes = tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_nodeSpecs, "f").slice();
        const marks = tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_markSpecs, "f").slice();
        const plugins = tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_plugins, "f").slice();
        const actions = tslib_1.__classPrivateFieldGet(this, _ExtensionBuilder_actions, "f").slice();
        return {
            configureMd: (md) => confMd.reduce((pMd, cb) => cb(pMd), md),
            nodes: () => {
                let map = orderedmap_1.default.from({});
                for (const [key, cb] of nodes) {
                    map = map.addToEnd(key, cb());
                }
                return map;
            },
            marks: () => {
                let map = orderedmap_1.default.from({});
                for (const [key, cb] of marks) {
                    map = map.addToEnd(key, cb());
                }
                return map;
            },
            plugins: (deps) => {
                // TODO: sort plugins here after WIKI-16660
                return plugins.reduce((acc, { cb, priority }) => {
                    const res = cb(deps);
                    if (Array.isArray(res))
                        acc.push(...res.map((plugin) => ({ plugin, priority })));
                    else
                        acc.push({ plugin: res, priority });
                    return acc;
                }, []);
            },
            actions: (deps) => actions.reduce((obj, [name, cb]) => {
                obj[name] = cb(deps);
                return obj;
            }, {}),
        };
    }
}
exports.ExtensionBuilder = ExtensionBuilder;
_ExtensionBuilder_confMdCbs = new WeakMap(), _ExtensionBuilder_nodeSpecs = new WeakMap(), _ExtensionBuilder_markSpecs = new WeakMap(), _ExtensionBuilder_plugins = new WeakMap(), _ExtensionBuilder_actions = new WeakMap();
ExtensionBuilder.PluginPriority = PluginPriority;
