"use strict";
var _ExtensionsManager_schemaRegistry, _ExtensionsManager_parserRegistry, _ExtensionsManager_serializerRegistry, _ExtensionsManager_nodeViewCreators, _ExtensionsManager_markViewCreators, _ExtensionsManager_md, _ExtensionsManager_extensions, _ExtensionsManager_deps, _ExtensionsManager_plugins, _ExtensionsManager_actions, _ExtensionsManager_nodeViews, _ExtensionsManager_markViews;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionsManager = void 0;
const tslib_1 = require("tslib");
const ActionsManager_1 = require("./ActionsManager");
const ParserTokensRegistry_1 = require("./ParserTokensRegistry");
const SchemaSpecRegistry_1 = require("./SchemaSpecRegistry");
const SerializerTokensRegistry_1 = require("./SerializerTokensRegistry");
class ExtensionsManager {
    constructor({ md, extensions }) {
        _ExtensionsManager_schemaRegistry.set(this, new SchemaSpecRegistry_1.SchemaSpecRegistry());
        _ExtensionsManager_parserRegistry.set(this, new ParserTokensRegistry_1.ParserTokensRegistry());
        _ExtensionsManager_serializerRegistry.set(this, new SerializerTokensRegistry_1.SerializerTokensRegistry());
        _ExtensionsManager_nodeViewCreators.set(this, new Map());
        _ExtensionsManager_markViewCreators.set(this, new Map());
        _ExtensionsManager_md.set(this, void 0);
        _ExtensionsManager_extensions.set(this, void 0);
        _ExtensionsManager_deps.set(this, void 0);
        _ExtensionsManager_plugins.set(this, []);
        _ExtensionsManager_actions.set(this, {});
        _ExtensionsManager_nodeViews.set(this, {});
        _ExtensionsManager_markViews.set(this, {});
        this.processNode = (name, { spec, fromYfm, toYfm, view }) => {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_schemaRegistry, "f").addNode(name, spec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").addToken(fromYfm.tokenName || name, fromYfm.tokenSpec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_serializerRegistry, "f").addNode(name, toYfm);
            if (view) {
                tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViewCreators, "f").set(name, view);
            }
        };
        this.processMark = (name, { spec, fromYfm, toYfm, view }) => {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_schemaRegistry, "f").addMark(name, spec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").addToken(fromYfm.tokenName || name, fromYfm.tokenSpec);
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_serializerRegistry, "f").addMark(name, toYfm);
            if (view) {
                tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViewCreators, "f").set(name, view);
            }
        };
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_md, md, "f");
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_extensions, extensions, "f");
    }
    static process(md, extensions) {
        return new this({ md, extensions }).build();
    }
    build() {
        this.processExtensions();
        this.createDeps();
        this.createDerived();
        return Object.assign(Object.assign({}, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f")), { actions: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f").actions, rawActions: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_actions, "f"), plugins: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_plugins, "f"), nodeViews: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViews, "f"), markViews: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViews, "f") });
    }
    buildDeps() {
        this.processExtensions();
        this.createDeps();
        return tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f");
    }
    processExtensions() {
        for (const ext of tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_extensions, "f")) {
            tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_md, ext.configureMd(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_md, "f")), "f");
            ext.nodes().forEach(this.processNode);
            ext.marks().forEach(this.processMark);
        }
    }
    createDeps() {
        const schema = tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_schemaRegistry, "f").createSchema();
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_deps, {
            schema,
            actions: new ActionsManager_1.ActionsManager(),
            parser: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_parserRegistry, "f").createParser(schema, tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_md, "f")),
            serializer: tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_serializerRegistry, "f").createSerializer(),
        }, "f");
    }
    createDerived() {
        const plugins = [];
        for (const ext of tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_extensions, "f")) {
            plugins.push(...ext.plugins(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f")));
            Object.assign(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_actions, "f"), ext.actions(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f")));
        }
        // TODO: move sorting to ExtensionBuilder after WIKI-16660
        tslib_1.__classPrivateFieldSet(this, _ExtensionsManager_plugins, plugins.sort((a, b) => b.priority - a.priority).map((item) => item.plugin), "f");
        for (const [name, view] of tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViewCreators, "f")) {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_nodeViews, "f")[name] = view(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f"));
        }
        for (const [name, view] of tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViewCreators, "f")) {
            tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_markViews, "f")[name] = view(tslib_1.__classPrivateFieldGet(this, _ExtensionsManager_deps, "f"));
        }
    }
}
exports.ExtensionsManager = ExtensionsManager;
_ExtensionsManager_schemaRegistry = new WeakMap(), _ExtensionsManager_parserRegistry = new WeakMap(), _ExtensionsManager_serializerRegistry = new WeakMap(), _ExtensionsManager_nodeViewCreators = new WeakMap(), _ExtensionsManager_markViewCreators = new WeakMap(), _ExtensionsManager_md = new WeakMap(), _ExtensionsManager_extensions = new WeakMap(), _ExtensionsManager_deps = new WeakMap(), _ExtensionsManager_plugins = new WeakMap(), _ExtensionsManager_actions = new WeakMap(), _ExtensionsManager_nodeViews = new WeakMap(), _ExtensionsManager_markViews = new WeakMap();
