"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseSchemaE = exports.BaseSchema = exports.toParagraph = exports.pType = exports.BaseNode = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_utils_1 = require("prosemirror-utils");
const core_1 = require("../../../core");
const schema_1 = require("../../../utils/schema");
var BaseNode;
(function (BaseNode) {
    BaseNode["Doc"] = "doc";
    BaseNode["Text"] = "text";
    BaseNode["Paragraph"] = "paragraph";
})(BaseNode = exports.BaseNode || (exports.BaseNode = {}));
exports.pType = (0, schema_1.nodeTypeFactory)(BaseNode.Paragraph);
const pAction = 'toParagraph';
const toParagraph = (state, dispatch) => (0, prosemirror_commands_1.setBlockType)((0, exports.pType)(state.schema))(state, dispatch);
exports.toParagraph = toParagraph;
const BaseSchema = (builder, opts) => {
    const { paragraphKey, paragraphPlaceholder } = opts;
    builder
        .addNode(BaseNode.Doc, () => ({
        spec: {
            content: 'block+',
        },
        fromYfm: { tokenSpec: { name: BaseNode.Doc, type: 'block', ignore: true } },
        toYfm: () => {
            throw new Error('Unexpected toYfm() call on doc node');
        },
    }))
        .addNode(BaseNode.Text, () => ({
        spec: {
            group: 'inline',
        },
        fromYfm: { tokenSpec: { name: BaseNode.Text, type: 'node', ignore: true } },
        toYfm: (state, node, parent) => {
            const { escapeText } = parent.type.spec;
            state.text(node.text, escapeText !== null && escapeText !== void 0 ? escapeText : !state.isAutolink);
        },
    }))
        .addNode(BaseNode.Paragraph, () => ({
        spec: {
            content: '(text | inline)*',
            group: 'block',
            parseDOM: [{ tag: 'p' }],
            toDOM() {
                return ['p', 0];
            },
            placeholder: paragraphPlaceholder
                ? {
                    content: paragraphPlaceholder,
                    alwaysVisible: false,
                }
                : undefined,
        },
        fromYfm: { tokenSpec: { name: BaseNode.Paragraph, type: 'block' } },
        toYfm: (state, node) => {
            state.renderInline(node);
            state.closeBlock(node);
        },
    }));
    if (paragraphKey) {
        builder.addKeymap(({ schema }) => ({ [paragraphKey]: (0, prosemirror_commands_1.setBlockType)((0, exports.pType)(schema)) }));
    }
    builder.addAction(pAction, ({ schema }) => {
        const p = (0, exports.pType)(schema);
        const cmd = (0, prosemirror_commands_1.setBlockType)(p);
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(p)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
};
exports.BaseSchema = BaseSchema;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.BaseSchemaE = (0, core_1.createExtension)((b, o) => b.use(exports.BaseSchema, o !== null && o !== void 0 ? o : {}));
