"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Placeholder = exports.createPlaceholder = exports.getPlaceholderContent = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_view_1 = require("prosemirror-view");
const prosemirror_utils_1 = require("prosemirror-utils");
const classname_1 = require("../../../classname");
const nodes_1 = require("../../../utils/nodes");
const selection_1 = require("../../../utils/selection");
const getPlaceholderContent = (node) => {
    var _a;
    const content = ((_a = node.type.spec.placeholder) === null || _a === void 0 ? void 0 : _a.content) || '';
    return typeof content === 'function' ? content(node) : content;
};
exports.getPlaceholderContent = getPlaceholderContent;
const getPlaceholderPluginKeys = (schema) => {
    var _a;
    const pluginKeys = [];
    for (const node in schema.nodes) {
        if (schema.nodes[node]) {
            const spec = schema.nodes[node].spec;
            if ((_a = spec.placeholder) === null || _a === void 0 ? void 0 : _a.customPlugin) {
                pluginKeys.push(spec.placeholder.customPlugin);
            }
        }
    }
    return pluginKeys;
};
const b = (0, classname_1.cn)('placeholder');
const createPlaceholder = (node, focus) => {
    const placeholder = document.createElement('div');
    placeholder.classList.add(...b({ focus }).split(' '));
    placeholder.textContent = (0, exports.getPlaceholderContent)(node);
    return placeholder;
};
exports.createPlaceholder = createPlaceholder;
const placeholderNeeded = (node) => {
    const childrenWithPlaceholderVisible = (0, prosemirror_utils_1.findChildren)(node, (n) => { var _a; return Boolean((_a = n.type.spec.placeholder) === null || _a === void 0 ? void 0 : _a.alwaysVisible); });
    return ((0, nodes_1.isNodeEmpty)(node) &&
        // Если есть дочерние ноды у которых есть постоянный плейсхолдер - отдаем им приоритет
        !childrenWithPlaceholderVisible.length);
};
const addDecoration = (decorationsMap, node, pos, cursorPos) => {
    const placeholderSpec = node.type.spec.placeholder;
    const decorationPosition = pos + node.childCount + 1;
    // Не добавляем декорацию если на этой позиции уже есть плейсхолдер
    if (!placeholderSpec || decorationsMap[decorationPosition])
        return;
    if (placeholderSpec.customPlugin) {
        decorationsMap[decorationPosition] = placeholderSpec.customPlugin;
        return;
    }
    decorationsMap[decorationPosition] = prosemirror_view_1.Decoration.widget(decorationPosition, (0, exports.createPlaceholder)(node, decorationPosition === cursorPos));
};
const Placeholder = (builder) => {
    builder.addPlugin(() => new prosemirror_state_1.Plugin({
        props: {
            decorations: (state) => {
                var _a;
                const decorationsMap = {};
                const { selection } = state;
                const cursorPos = (0, selection_1.isTextSelection)(selection)
                    ? (_a = selection.$cursor) === null || _a === void 0 ? void 0 : _a.pos
                    : null;
                getPlaceholderPluginKeys(state.schema).forEach((f) => {
                    var _a;
                    // Используем find потому что при помощи него можно проитерировать по DecorationSet.
                    (_a = f.getState(state)) === null || _a === void 0 ? void 0 : _a.find(undefined, undefined, (spec) => {
                        decorationsMap[spec.pos] = f;
                        return false;
                    });
                });
                // Рисуем плэйсхолдеры для всех нод у которых плэйсхолдер alwaysVisible
                const decorate = (node, pos) => {
                    const placeholderSpec = node.type.spec.placeholder;
                    if (placeholderSpec &&
                        placeholderSpec.alwaysVisible &&
                        placeholderNeeded(node)) {
                        addDecoration(decorationsMap, node, pos, cursorPos);
                    }
                };
                state.doc.descendants(decorate);
                const parentNode = (0, prosemirror_utils_1.findParentNodeClosestToPos)(state.selection.$from, (node) => {
                    return Boolean(node.type.spec.placeholder);
                });
                const placeholderSpec = parentNode === null || parentNode === void 0 ? void 0 : parentNode.node.type.spec.placeholder;
                // Дорисовываем плэйсхолдер, если его нужно отрисовать на месте курсора
                if (parentNode &&
                    placeholderNeeded(parentNode.node) &&
                    placeholderSpec &&
                    !placeholderSpec.alwaysVisible) {
                    addDecoration(decorationsMap, parentNode.node, parentNode.pos, cursorPos);
                }
                const decorations = Object.values(decorationsMap).filter((decoration) => !(decoration instanceof prosemirror_state_1.PluginKey));
                return prosemirror_view_1.DecorationSet.create(state.doc, decorations);
            },
        },
    }));
};
exports.Placeholder = Placeholder;
