"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toggleQuote = exports.selectQuoteBeforeCursor = exports.liftFromQuote = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_state_1 = require("prosemirror-state");
const selection_1 = require("../../../utils/selection");
const const_1 = require("./const");
require("../../../types/spec");
function getCursor(sel) {
    if ((0, selection_1.isTextSelection)(sel))
        return sel.$cursor;
    return undefined;
}
const liftFromQuote = (state, dispatch) => {
    const $cursor = getCursor(state.selection);
    if (!$cursor || $cursor.parentOffset > 0 || $cursor.node(-1).type !== (0, const_1.bqType)(state.schema)) {
        return false;
    }
    return (0, prosemirror_commands_1.lift)(state, dispatch);
};
exports.liftFromQuote = liftFromQuote;
const selectQuoteBeforeCursor = (state, dispatch) => {
    const $cursor = getCursor(state.selection);
    if (!$cursor || $cursor.parentOffset > 0)
        return false;
    const index = $cursor.index(-1);
    const nodeBefore = $cursor.node(-1).maybeChild(index - 1);
    if (!nodeBefore || nodeBefore.type !== (0, const_1.bqType)(state.schema))
        return false;
    const beforePos = $cursor.before();
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.setSelection(prosemirror_state_1.NodeSelection.create(state.doc, beforePos - nodeBefore.nodeSize)));
    return true;
};
exports.selectQuoteBeforeCursor = selectQuoteBeforeCursor;
const toggleQuote = (state, dispatch) => {
    const { selection } = state;
    const qType = (0, const_1.bqType)(state.schema);
    if (!(0, selection_1.isTextSelection)(selection) || !selection.$cursor)
        return (0, prosemirror_commands_1.wrapIn)(qType)(state, dispatch);
    const { $cursor } = selection;
    let { depth } = $cursor;
    while (depth >= 0) {
        const node = $cursor.node(depth);
        const nodeSpec = node.type.spec;
        if (!nodeSpec.complex || nodeSpec.complex === 'root') {
            const targetDepth = depth - 1;
            const range = getBlockRange($cursor, depth);
            if ($cursor.node(targetDepth).type === qType) {
                dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.lift(range, targetDepth - 1).scrollIntoView());
            }
            else {
                dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.wrap(range, [{ type: qType }]).scrollIntoView());
            }
            return true;
        }
        depth--;
    }
    return false;
};
exports.toggleQuote = toggleQuote;
function getBlockRange($pos, depth) {
    const { doc } = $pos;
    const $before = doc.resolve($pos.before(depth));
    const $after = doc.resolve($pos.after(depth));
    return $before.blockRange($after);
}
