"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockquoteE = exports.Blockquote = exports.blockquote = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const prosemirror_utils_1 = require("prosemirror-utils");
const core_1 = require("../../../core");
const commands_1 = require("./commands");
const const_1 = require("./const");
Object.defineProperty(exports, "blockquote", { enumerable: true, get: function () { return const_1.blockquote; } });
const bqAction = 'quote';
const Blockquote = (builder, opts) => {
    builder.addNode(const_1.blockquote, () => ({
        spec: {
            content: 'block+',
            group: 'block',
            defining: true,
            parseDOM: [{ tag: 'blockquote' }],
            toDOM() {
                return ['blockquote', 0];
            },
        },
        fromYfm: { tokenSpec: { name: const_1.blockquote, type: 'block' } },
        toYfm: (state, node) => {
            state.wrapBlock('> ', null, node, () => state.renderContent(node));
        },
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.qouteKey) {
        const { qouteKey } = opts;
        builder.addKeymap(({ schema }) => ({ [qouteKey]: (0, prosemirror_commands_1.wrapIn)((0, const_1.bqType)(schema)) }));
    }
    builder.addKeymap(() => ({
        Backspace: (0, prosemirror_commands_1.chainCommands)(commands_1.liftFromQuote, commands_1.selectQuoteBeforeCursor),
    }));
    builder.addInputRules(({ schema }) => ({ rules: [blockQuoteRule((0, const_1.bqType)(schema))] }));
    builder.addAction(bqAction, ({ schema }) => {
        const bq = (0, const_1.bqType)(schema);
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(bq)(state.selection),
            isEnable: commands_1.toggleQuote,
            run: commands_1.toggleQuote,
        };
    });
};
exports.Blockquote = Blockquote;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.BlockquoteE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Blockquote, o));
/**
 * Given a blockquote node type, returns an input rule that turns `"> "`
 * at the start of a textblock into a blockquote.
 */
function blockQuoteRule(nodeType) {
    return (0, prosemirror_inputrules_1.wrappingInputRule)(/^\s*>\s$/, nodeType);
}
