"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BoldE = exports.Bold = exports.bold = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const actions_1 = require("../../../utils/actions");
const core_1 = require("../../../core");
const schema_1 = require("../../../utils/schema");
const inputrules_1 = require("../../../utils/inputrules");
exports.bold = 'strong';
const bAction = 'bold';
const bType = (0, schema_1.markTypeFactory)(exports.bold);
const Bold = (builder, opts) => {
    builder
        .addMark(exports.bold, () => ({
        spec: {
            parseDOM: [{ tag: 'strong' }],
            toDOM() {
                return ['strong'];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: exports.bold,
                type: 'mark',
            },
        },
        toYfm: { open: '**', close: '**', mixable: true, expelEnclosingWhitespace: true },
    }))
        .addAction(bAction, ({ schema }) => (0, actions_1.createToggleMarkAction)(bType(schema)));
    if (opts === null || opts === void 0 ? void 0 : opts.boldKey) {
        const { boldKey } = opts;
        builder.addKeymap(({ schema }) => ({ [boldKey]: (0, prosemirror_commands_1.toggleMark)(bType(schema)) }));
    }
    builder.addInputRules(({ schema }) => ({
        rules: [
            (0, inputrules_1.markInputRule)({ open: '**', close: '**', ignoreBetween: '*' }, bType(schema)),
            (0, inputrules_1.markInputRule)({ open: '__', close: '__', ignoreBetween: '_' }, bType(schema)),
        ],
    }));
};
exports.Bold = Bold;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.BoldE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Bold, o));
