"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Breaks = exports.sbType = exports.hbType = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const platform_1 = require("../../../utils/platform");
const schema_1 = require("../../../utils/schema");
const hardBreak = 'hard_break';
const softBreak = 'soft_break';
exports.hbType = (0, schema_1.nodeTypeFactory)(hardBreak);
exports.sbType = (0, schema_1.nodeTypeFactory)(softBreak);
const Breaks = (builder, opts) => {
    builder.addNode(hardBreak, () => ({
        spec: {
            inline: true,
            group: 'inline',
            selectable: false,
            parseDOM: [{ tag: 'br' }],
            toDOM() {
                return ['br'];
            },
        },
        fromYfm: { tokenName: 'hardbreak', tokenSpec: { name: hardBreak, type: 'node' } },
        toYfm: (state, node, parent, index) => {
            for (let i = index + 1; i < parent.childCount; i++) {
                if (parent.child(i).type !== node.type) {
                    state.write('\\\n');
                    return;
                }
            }
        },
    }));
    // TODO: should we handle softbreak differently at different md.options.breaks setting?
    // we can safely convert softbreak into hardbreak,
    // but in this case non-edited markup will always be changed – a backspash will be added
    builder.addNode(softBreak, () => ({
        spec: {
            inline: true,
            group: 'inline',
            selectable: false,
            toDOM() {
                return ['br'];
            },
        },
        fromYfm: { tokenName: 'softbreak', tokenSpec: { name: softBreak, type: 'node' } },
        toYfm: (state, node, parent, index) => {
            for (let i = index + 1; i < parent.childCount; i++) {
                if (parent.child(i).type !== node.type) {
                    state.write('\n');
                    return;
                }
            }
        },
    }));
    builder.addKeymap(({ schema }) => {
        const cmd = addBr(((opts === null || opts === void 0 ? void 0 : opts.preferredBreak) === 'soft' ? exports.sbType : exports.hbType)(schema));
        const keys = {
            'Shift-Enter': cmd,
        };
        if ((0, platform_1.isMac)()) {
            keys['Ctrl-Enter'] = cmd;
        }
        return keys;
    });
};
exports.Breaks = Breaks;
const addBr = (br) => (0, prosemirror_commands_1.chainCommands)(prosemirror_commands_1.exitCode, (state, dispatch) => {
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.replaceSelectionWith(br.create()).scrollIntoView());
    return true;
});
