"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeE = exports.Code = exports.codeType = exports.code = void 0;
const tslib_1 = require("tslib");
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_codemark_1 = tslib_1.__importDefault(require("prosemirror-codemark"));
const actions_1 = require("../../../utils/actions");
const core_1 = require("../../../core");
const schema_1 = require("../../../utils/schema");
exports.code = 'code';
const codeAction = 'code';
exports.codeType = (0, schema_1.markTypeFactory)(exports.code);
const Code = (builder, opts) => {
    builder
        .addMark(exports.code, () => ({
        spec: {
            code: true,
            parseDOM: [{ tag: 'code' }],
            toDOM() {
                return ['code'];
            },
        },
        toYfm: {
            open(_state, _mark, parent, index) {
                return backticksFor(parent.child(index), -1);
            },
            close(_state, _mark, parent, index) {
                return backticksFor(parent.child(index - 1), 1);
            },
            escape: false,
        },
        fromYfm: {
            tokenSpec: { name: exports.code, type: 'mark', noCloseToken: true },
            tokenName: 'code_inline',
        },
    }))
        .addAction(codeAction, ({ schema }) => (0, actions_1.createToggleMarkAction)((0, exports.codeType)(schema)));
    if (opts === null || opts === void 0 ? void 0 : opts.codeKey) {
        const { codeKey } = opts;
        builder.addKeymap(({ schema }) => ({ [codeKey]: (0, prosemirror_commands_1.toggleMark)((0, exports.codeType)(schema)) }));
    }
    builder
        // codemark adds inputRule for `code`,
        // adds fake cursor behaviour when crossing inline_code boundaries,
        // wrap current selected text to inline_code when '`' was pressed.
        // See demo: https://curvenote.github.io/prosemirror-codemark/
        .addPlugin(({ schema }) => (0, prosemirror_codemark_1.default)({ markType: (0, exports.codeType)(schema) }));
};
exports.Code = Code;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.CodeE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Code, o));
function backticksFor(node, side) {
    const ticks = /`+/g;
    let m;
    let len = 0;
    if (node.isText)
        while ((m = ticks.exec(node.text || '')))
            len = Math.max(len, m[0].length);
    let result = len > 0 && side > 0 ? ' `' : '`';
    for (let i = 0; i < len; i++)
        result += '`';
    if (len > 0 && side < 0)
        result += ' ';
    return result;
}
