"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeadingE = exports.Heading = exports.hType = exports.HeadingAction = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const core_1 = require("../../../core");
const actions_1 = require("./actions");
const const_1 = require("./const");
const utils_1 = require("./utils");
const commands_1 = require("./commands");
var const_2 = require("./const");
Object.defineProperty(exports, "HeadingAction", { enumerable: true, get: function () { return const_2.HeadingAction; } });
var utils_2 = require("./utils");
Object.defineProperty(exports, "hType", { enumerable: true, get: function () { return utils_2.hType; } });
const DEFAULT_PLACEHOLDER = (node) => 'Heading ' + node.attrs[const_1.lvlAttr];
const Heading = (builder, opts) => {
    const { headingPlaceholder } = opts !== null && opts !== void 0 ? opts : {};
    builder.addNode(const_1.heading, () => ({
        spec: {
            attrs: { [const_1.lvlAttr]: { default: 1 } },
            content: '(text | inline)*',
            group: 'block',
            defining: true,
            parseDOM: [
                { tag: 'h1', attrs: { [const_1.lvlAttr]: 1 } },
                { tag: 'h2', attrs: { [const_1.lvlAttr]: 2 } },
                { tag: 'h3', attrs: { [const_1.lvlAttr]: 3 } },
                { tag: 'h4', attrs: { [const_1.lvlAttr]: 4 } },
                { tag: 'h5', attrs: { [const_1.lvlAttr]: 5 } },
                { tag: 'h6', attrs: { [const_1.lvlAttr]: 6 } },
            ],
            toDOM(node) {
                return ['h' + node.attrs[const_1.lvlAttr], 0];
            },
            placeholder: {
                content: headingPlaceholder !== null && headingPlaceholder !== void 0 ? headingPlaceholder : DEFAULT_PLACEHOLDER,
                alwaysVisible: true,
            },
        },
        fromYfm: {
            tokenSpec: {
                name: const_1.heading,
                type: 'block',
                getAttrs: (tok) => ({ [const_1.lvlAttr]: Number(tok.tag.slice(1)) }),
            },
        },
        toYfm: (state, node) => {
            state.write(state.repeat('#', node.attrs[const_1.lvlAttr]) + ' ');
            state.renderInline(node);
            state.closeBlock(node);
        },
    }));
    builder
        .addKeymap(({ schema }) => {
        const { h1Key, h2Key, h3Key, h4Key, h5Key, h6Key } = opts !== null && opts !== void 0 ? opts : {};
        const cmd4lvl = (level) => (0, prosemirror_commands_1.setBlockType)((0, utils_1.hType)(schema), { [const_1.lvlAttr]: level });
        const bindings = { Backspace: commands_1.resetHeading };
        if (h1Key)
            bindings[h1Key] = cmd4lvl(1);
        if (h2Key)
            bindings[h2Key] = cmd4lvl(2);
        if (h3Key)
            bindings[h3Key] = cmd4lvl(3);
        if (h4Key)
            bindings[h4Key] = cmd4lvl(4);
        if (h5Key)
            bindings[h5Key] = cmd4lvl(5);
        if (h6Key)
            bindings[h6Key] = cmd4lvl(6);
        return bindings;
    })
        .addInputRules(({ schema }) => ({ rules: [(0, utils_1.headingRule)((0, utils_1.hType)(schema), 6)] }));
    builder
        .addAction(const_1.HeadingAction.ToH1, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 1))
        .addAction(const_1.HeadingAction.ToH2, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 2))
        .addAction(const_1.HeadingAction.ToH3, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 3))
        .addAction(const_1.HeadingAction.ToH4, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 4))
        .addAction(const_1.HeadingAction.ToH5, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 5))
        .addAction(const_1.HeadingAction.ToH6, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 6));
};
exports.Heading = Heading;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.HeadingE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Heading, o));
