"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HorizontalRuleE = exports.HorizontalRule = exports.markupAttr = exports.horizontalRule = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const core_1 = require("../../../core");
const schema_1 = require("../../../utils/schema");
exports.horizontalRule = 'horizontal_rule';
const hrAction = 'hRule';
exports.markupAttr = 'markup';
const hrType = (0, schema_1.nodeTypeFactory)(exports.horizontalRule);
const HorizontalRule = (builder) => {
    builder.addNode(exports.horizontalRule, () => ({
        spec: {
            attrs: { [exports.markupAttr]: { default: '---' } },
            group: 'block',
            parseDOM: [{ tag: 'hr' }],
            toDOM() {
                return ['div', ['hr']];
            },
        },
        fromYfm: {
            tokenName: 'hr',
            tokenSpec: {
                name: exports.horizontalRule,
                type: 'node',
                getAttrs: (token) => ({ [exports.markupAttr]: token.markup }),
            },
        },
        toYfm: (state, node) => {
            state.write(node.attrs[exports.markupAttr]);
            state.closeBlock(node);
        },
    }));
    builder.addAction(hrAction, ({ schema }) => {
        const cmd = addHr(hrType(schema));
        return {
            isActive: (state) => isHrSelection(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
};
exports.HorizontalRule = HorizontalRule;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.HorizontalRuleE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.HorizontalRule, o));
const addHr = (hr) => (state, dispatch) => {
    if (!isHrSelection(state.selection)) {
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.replaceSelectionWith(hr.create()).scrollIntoView());
    }
    return true;
};
function isHrSelection(selection) {
    return selection instanceof prosemirror_state_1.NodeSelection && selection.node.type.name === exports.horizontalRule;
}
