"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinkE = exports.Link = exports.LinkAttr = exports.linkType = exports.link = exports.normalizeUrlFactory = void 0;
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const core_1 = require("../../../core");
const schema_1 = require("../../../utils/schema");
const actions_1 = require("./actions");
var utils_1 = require("./utils");
Object.defineProperty(exports, "normalizeUrlFactory", { enumerable: true, get: function () { return utils_1.normalizeUrlFactory; } });
exports.link = 'link';
const linkAction = 'link';
exports.linkType = (0, schema_1.markTypeFactory)(exports.link);
var LinkAttr;
(function (LinkAttr) {
    LinkAttr["Href"] = "href";
    LinkAttr["Title"] = "title";
    // tech attributes
    LinkAttr["IsPlaceholder"] = "is-placeholder";
    LinkAttr["RawLink"] = "raw-link";
})(LinkAttr = exports.LinkAttr || (exports.LinkAttr = {}));
const Link = (builder) => {
    builder
        .addMark(exports.link, () => ({
        spec: {
            attrs: {
                [LinkAttr.Href]: {},
                [LinkAttr.Title]: { default: null },
                [LinkAttr.IsPlaceholder]: { default: false },
                [LinkAttr.RawLink]: { default: false },
            },
            inclusive: false,
            parseDOM: [
                {
                    tag: 'a[href]',
                    getAttrs(dom) {
                        return {
                            href: dom.getAttribute(LinkAttr.Href),
                            title: dom.getAttribute(LinkAttr.Title),
                        };
                    },
                },
            ],
            toDOM(node) {
                return ['a', node.attrs];
            },
        },
        toYfm: {
            open(state, mark, parent, index) {
                state.isAutolink = isPlainURL(mark, parent, index, 1);
                if (state.isAutolink) {
                    if (mark.attrs[LinkAttr.RawLink])
                        return '';
                    return '<';
                }
                return '[';
            },
            close(state, mark) {
                if (state.isAutolink) {
                    state.isAutolink = undefined;
                    if (mark.attrs[LinkAttr.RawLink])
                        return '';
                    return '>';
                }
                state.isAutolink = undefined;
                return ('](' +
                    mark.attrs[LinkAttr.Href] +
                    (mark.attrs[LinkAttr.Title]
                        ? ' ' + state.quote(mark.attrs[LinkAttr.Title])
                        : '') +
                    ')');
            },
        },
        fromYfm: {
            tokenSpec: {
                name: exports.link,
                type: 'mark',
                getAttrs: (tok) => ({
                    href: tok.attrGet('href'),
                    title: tok.attrGet('title') || null,
                }),
            },
        },
    }))
        .addAction(linkAction, (deps) => (0, actions_1.linkCommand)((0, exports.linkType)(deps.schema), deps))
        .addInputRules(({ schema }) => ({ rules: [linkInputRule((0, exports.linkType)(schema))] }));
};
exports.Link = Link;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.LinkE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Link, o));
function isPlainURL(link, parent, index, side) {
    if (link.attrs.title || !/^\w+:/.test(link.attrs[LinkAttr.Href]))
        return false;
    const content = parent.child(index + (side < 0 ? -1 : 0));
    if (!content.isText ||
        content.text !== link.attrs[LinkAttr.Href] ||
        content.marks[content.marks.length - 1] !== link)
        return false;
    if (index === (side < 0 ? 1 : parent.childCount - 1))
        return true;
    const next = parent.child(index + (side < 0 ? -2 : 1));
    return !link.isInSet(next.marks);
}
// TODO: think about generalizing with markInputRule
function linkInputRule(markType) {
    return new prosemirror_inputrules_1.InputRule(/\[(.+)]\((\S+)\)\s$/, (state, match, start, end) => {
        // handle the rule only if is start of line or there is a space before "open" symbols
        if (match.index > 0) {
            const re = match;
            if (re.input[re.index - 1] !== ' ')
                return null;
        }
        const [okay, alt, href] = match;
        const { tr } = state;
        if (okay) {
            tr.replaceWith(start, end, markType.schema.text(alt)).addMark(start, start + alt.length, markType.create({ href }));
        }
        return tr;
    });
}
