"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListsE = exports.Lists = void 0;
const prosemirror_schema_list_1 = require("prosemirror-schema-list");
const core_1 = require("../../../core");
const actions_1 = require("./actions");
const const_1 = require("./const");
const fromYfm_1 = require("./fromYfm");
const spec_1 = require("./spec");
const toYfm_1 = require("./toYfm");
const inputrules_1 = require("./inputrules");
const utils_1 = require("./utils");
const commands_1 = require("./commands");
const Lists = (builder, opts) => {
    builder
        .addNode(const_1.ListNode.ListItem, () => ({
        spec: spec_1.spec[const_1.ListNode.ListItem],
        toYfm: toYfm_1.toYfm[const_1.ListNode.ListItem],
        fromYfm: { tokenSpec: fromYfm_1.fromYfm[const_1.ListNode.ListItem] },
    }))
        .addNode(const_1.ListNode.BulletList, () => ({
        spec: spec_1.spec[const_1.ListNode.BulletList],
        toYfm: toYfm_1.toYfm[const_1.ListNode.BulletList],
        fromYfm: { tokenSpec: fromYfm_1.fromYfm[const_1.ListNode.BulletList] },
    }))
        .addNode(const_1.ListNode.OrderedList, () => ({
        spec: spec_1.spec[const_1.ListNode.OrderedList],
        toYfm: toYfm_1.toYfm[const_1.ListNode.OrderedList],
        fromYfm: { tokenSpec: fromYfm_1.fromYfm[const_1.ListNode.OrderedList] },
    }));
    builder.addKeymap(({ schema }) => {
        const { ulKey, olKey } = opts !== null && opts !== void 0 ? opts : {};
        const bindings = {};
        if (ulKey)
            bindings[ulKey] = (0, commands_1.toList)((0, utils_1.blType)(schema));
        if (olKey)
            bindings[olKey] = (0, commands_1.toList)((0, utils_1.olType)(schema));
        return Object.assign({ Enter: (0, prosemirror_schema_list_1.splitListItem)((0, utils_1.liType)(schema)), Tab: (0, prosemirror_schema_list_1.sinkListItem)((0, utils_1.liType)(schema)), 'Shift-Tab': (0, prosemirror_schema_list_1.liftListItem)((0, utils_1.liType)(schema)), 'Mod-[': (0, prosemirror_schema_list_1.liftListItem)((0, utils_1.liType)(schema)), 'Mod-]': (0, prosemirror_schema_list_1.sinkListItem)((0, utils_1.liType)(schema)) }, bindings);
    });
    builder.use(inputrules_1.ListsInputRulesExtension, { bulletListInputRule: opts === null || opts === void 0 ? void 0 : opts.ulInputRules });
    builder
        .addAction(const_1.ListAction.ToBulletList, actions_1.actions.toBulletList)
        .addAction(const_1.ListAction.ToOrderedList, actions_1.actions.toOrderedList)
        .addAction(const_1.ListAction.SinkListItem, actions_1.actions.sinkListItem)
        .addAction(const_1.ListAction.LiftListItem, actions_1.actions.liftListItem);
};
exports.Lists = Lists;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.ListsE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Lists, o));
