"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bulletListRule = exports.orderedListRule = exports.ListsInputRulesExtension = void 0;
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const utils_1 = require("./utils");
const ListsInputRulesExtension = (builder, options) => {
    builder.addInputRules(({ schema }) => {
        const rules = [orderedListRule((0, utils_1.olType)(schema))];
        if (options.bulletListInputRule !== false) {
            const blRule = bulletListRule((0, utils_1.blType)(schema), options.bulletListInputRule);
            if (blRule)
                rules.push(blRule);
        }
        return { rules };
    });
};
exports.ListsInputRulesExtension = ListsInputRulesExtension;
/**
 * Given a list node type, returns an input rule that turns a number
 * followed by a dot at the start of a textblock into an ordered list.
 */
function orderedListRule(nodeType) {
    return (0, prosemirror_inputrules_1.wrappingInputRule)(/^(\d+)\.\s$/, nodeType, (match) => ({ order: Number(match[1]) }), (match, node) => node.childCount + node.attrs.order === Number(match[1]));
}
exports.orderedListRule = orderedListRule;
/**
 * Given a list node type, returns an input rule that turns a bullet
 * (dash, plus, or asterisk) at the start of a textblock into a
 * bullet list.
 */
function bulletListRule(nodeType, config) {
    const bullets = [];
    if ((config === null || config === void 0 ? void 0 : config.dash) !== false)
        bullets.push('-');
    if ((config === null || config === void 0 ? void 0 : config.plus) !== false)
        bullets.push('+');
    if ((config === null || config === void 0 ? void 0 : config.asterisk) !== false)
        bullets.push('*');
    if (bullets.length === 0)
        return null;
    const regexp = new RegExp(`^\\s*([${bullets.join('')}])\\s$`); // same as /^\s*([-+*])\s$/
    return (0, prosemirror_inputrules_1.wrappingInputRule)(regexp, nodeType);
}
exports.bulletListRule = bulletListRule;
