"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StrikeE = exports.Strike = exports.strike = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const core_1 = require("../../../core");
const actions_1 = require("../../../utils/actions");
const schema_1 = require("../../../utils/schema");
const inputrules_1 = require("../../../utils/inputrules");
exports.strike = 'strike';
const sAction = 'strike';
const sType = (0, schema_1.markTypeFactory)(exports.strike);
const Strike = (builder, opts) => {
    builder
        .addMark(exports.strike, () => ({
        spec: {
            parseDOM: [{ tag: 'strike' }],
            toDOM() {
                return ['strike'];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: exports.strike,
                type: 'mark',
            },
            tokenName: 's',
        },
        toYfm: { open: '~~', close: '~~', mixable: true, expelEnclosingWhitespace: true },
    }))
        .addAction(sAction, ({ schema }) => (0, actions_1.createToggleMarkAction)(sType(schema)))
        .addInputRules(({ schema }) => ({
        rules: [(0, inputrules_1.markInputRule)({ open: '~~', close: '~~', ignoreBetween: '~' }, sType(schema))],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.strikeKey) {
        const { strikeKey } = opts;
        builder.addKeymap(({ schema }) => ({ [strikeKey]: (0, prosemirror_commands_1.toggleMark)(sType(schema)) }));
    }
};
exports.Strike = Strike;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.StrikeE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Strike, o));
