"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnderlineE = exports.Underline = exports.underline = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const core_1 = require("../../../core");
const actions_1 = require("../../../utils/actions");
const schema_1 = require("../../../utils/schema");
const inputrules_1 = require("../../../utils/inputrules");
const ins = require('markdown-it-ins');
exports.underline = 'ins';
const undAction = 'underline';
const undType = (0, schema_1.markTypeFactory)(exports.underline);
const Underline = (builder, opts) => {
    builder
        .configureMd((md) => md.use(ins))
        .addMark(exports.underline, () => ({
        spec: {
            parseDOM: [{ tag: 'ins' }],
            toDOM() {
                return ['ins'];
            },
        },
        toYfm: { open: '++', close: '++', mixable: true, expelEnclosingWhitespace: true },
        fromYfm: { tokenSpec: { name: exports.underline, type: 'mark' } },
    }))
        .addAction(undAction, ({ schema }) => (0, actions_1.createToggleMarkAction)(undType(schema)))
        .addInputRules(({ schema }) => ({
        rules: [(0, inputrules_1.markInputRule)({ open: '++', close: '++', ignoreBetween: '+' }, undType(schema))],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.underlineKey) {
        const { underlineKey } = opts;
        builder.addKeymap(({ schema }) => ({ [underlineKey]: (0, prosemirror_commands_1.toggleMark)(undType(schema)) }));
    }
};
exports.Underline = Underline;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.UnderlineE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Underline, o));
