"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckboxE = exports.Checkbox = void 0;
const tslib_1 = require("tslib");
const checkbox_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/plugins/checkbox"));
const core_1 = require("../../../core");
const prosemirror_utils_1 = require("prosemirror-utils");
const const_1 = require("./const");
const plugin_1 = require("./plugin");
const classname_1 = require("../../../classname");
const spec_1 = require("./spec");
const toYfm_1 = require("./toYfm");
const fromYfm_1 = require("./fromYfm");
const actions_1 = require("./actions");
const inputrules_1 = require("../../../utils/inputrules");
const utils_1 = require("./utils");
const BaseSchema_1 = require("../../base/BaseSchema");
const checkboxAction = 'addCheckbox';
const idPrefix = 'yfm-editor-checkbox';
const b = (0, classname_1.cn)('checkbox');
const Checkbox = (builder, opts) => {
    const spec = (0, spec_1.getSpec)(opts);
    builder
        .configureMd((md) => (0, checkbox_1.default)(md, { idPrefix, divClass: b() }))
        .addNode(const_1.CheckboxNode.Checkbox, () => ({
        spec: spec[const_1.CheckboxNode.Checkbox],
        toYfm: toYfm_1.toYfm[const_1.CheckboxNode.Checkbox],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.CheckboxNode.Checkbox],
        },
    }))
        .addNode(const_1.CheckboxNode.Input, () => ({
        spec: spec[const_1.CheckboxNode.Input],
        toYfm: toYfm_1.toYfm[const_1.CheckboxNode.Input],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.CheckboxNode.Input],
        },
        view: () => (node, view, getPos) => {
            const dom = document.createElement('input');
            for (const attr in node.attrs) {
                if (node.attrs[attr])
                    dom.setAttribute(attr, node.attrs[attr]);
            }
            dom.setAttribute('class', b('input'));
            dom.addEventListener('click', (e) => {
                const elem = e.target;
                const checkedAttr = elem.getAttribute('checked');
                const checked = checkedAttr ? '' : 'true';
                view.dispatch(view.state.tr.setNodeMarkup(getPos(), undefined, Object.assign(Object.assign({}, node.attrs), { checked })));
                elem.setAttribute('checked', checked);
            });
            return {
                dom,
                ignoreMutation: () => true,
                update: () => true,
                destroy() {
                    const resolved = view.state.doc.resolve(getPos());
                    if (resolved.parent.type.name === const_1.CheckboxNode.Checkbox &&
                        resolved.parent.lastChild) {
                        view.dispatch((0, prosemirror_utils_1.replaceParentNodeOfType)(resolved.parent.type, (0, BaseSchema_1.pType)(view.state.schema).create(resolved.parent.lastChild.content))(view.state.tr));
                    }
                    dom.remove();
                },
            };
        },
    }))
        .addNode(const_1.CheckboxNode.Label, () => ({
        spec: spec[const_1.CheckboxNode.Label],
        toYfm: toYfm_1.toYfm[const_1.CheckboxNode.Label],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.CheckboxNode.Label],
            tokenName: 'checkbox_label',
        },
    }))
        .addPlugin(plugin_1.keymapPlugin)
        .addAction(checkboxAction, () => (0, actions_1.addCheckbox)())
        .addInputRules(({ schema }) => ({
        rules: [
            (0, inputrules_1.nodeInputRule)(/^\[(\s?)\]\s$/, (0, utils_1.checkboxType)(schema).createAndFill({}, (0, utils_1.checkboxInputType)(schema).create()), 2),
        ],
    }));
};
exports.Checkbox = Checkbox;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.CheckboxE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.Checkbox, o));
