"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.keymapPlugin = exports.splitCheckbox = void 0;
const prosemirror_keymap_1 = require("prosemirror-keymap");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const BaseSchema_1 = require("../../base/BaseSchema");
const utils_1 = require("./utils");
const splitCheckbox = (state, dispatch) => {
    const { $from, $to } = state.selection;
    const { schema } = state;
    const label = (0, prosemirror_utils_1.findParentNodeOfType)((0, utils_1.checkboxLabelType)(schema))(state.selection);
    if (!label)
        return false;
    const checkbox = (0, prosemirror_utils_1.findParentNodeOfType)((0, utils_1.checkboxType)(schema))(state.selection);
    if (label.node.content.size === 0 && checkbox) {
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr
            .replaceWith(checkbox.pos, checkbox.pos + checkbox.node.nodeSize, (0, BaseSchema_1.pType)(schema).create())
            .scrollIntoView());
        return true;
    }
    if ($from.pos === $to.pos) {
        const { tr } = state;
        const node = (0, utils_1.checkboxType)(schema).create({}, [
            (0, utils_1.checkboxInputType)(schema).create(),
            (0, utils_1.checkboxLabelType)(schema).create({}, prosemirror_model_1.Fragment.from($from.parent.cut($from.parentOffset).content)),
        ]);
        tr.insert($from.after(), [node]);
        tr.replace(label.start + $from.parentOffset, label.pos + label.node.nodeSize);
        tr.setSelection(new prosemirror_state_1.TextSelection(tr.doc.resolve(tr.selection.$from.after() + 4)));
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr);
        return true;
    }
    return false;
};
exports.splitCheckbox = splitCheckbox;
const removeCheckbox = (state, dispatch) => {
    const label = (0, prosemirror_utils_1.findParentNodeOfType)((0, utils_1.checkboxLabelType)(state.schema))(state.selection);
    const checkbox = (0, prosemirror_utils_1.findParentNodeOfType)((0, utils_1.checkboxType)(state.schema))(state.selection);
    if (!label || !checkbox) {
        return false;
    }
    const idx = state.selection.from - label.pos - 2;
    if (idx < 0) {
        const { tr } = state;
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(tr
            .replaceWith(checkbox.pos, checkbox.pos + checkbox.node.nodeSize, (0, BaseSchema_1.pType)(state.schema).create(null, label.node.content))
            .setSelection(new prosemirror_state_1.TextSelection(tr.doc.resolve(state.selection.from - 2))));
        return true;
    }
    return false;
};
const keymapPlugin = () => (0, prosemirror_keymap_1.keymap)({
    Enter: exports.splitCheckbox,
    Backspace: removeCheckbox,
});
exports.keymapPlugin = keymapPlugin;
