"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorE = exports.Color = exports.Colors = exports.colorClassName = void 0;
const tslib_1 = require("tslib");
const markdown_it_color_1 = tslib_1.__importDefault(require("markdown-it-color"));
const prosemirror_commands_1 = require("prosemirror-commands");
const core_1 = require("../../../core");
const marks_1 = require("../../../utils/marks");
const schema_1 = require("../../../utils/schema");
const const_1 = require("./const");
const utils_1 = require("./utils");
const colorType = (0, schema_1.markTypeFactory)(const_1.color);
var const_2 = require("./const");
Object.defineProperty(exports, "colorClassName", { enumerable: true, get: function () { return const_2.className; } });
Object.defineProperty(exports, "Colors", { enumerable: true, get: function () { return const_2.Colors; } });
const Color = (builder) => {
    builder
        .configureMd((md) => md.use(markdown_it_color_1.default, { defaultClassName: const_1.className, inline: false }))
        .addMark(const_1.color, () => ({
        spec: {
            attrs: { [const_1.color]: {} },
            parseDOM: [
                {
                    tag: `span[${const_1.domColorAttr}]`,
                    getAttrs(node) {
                        return {
                            [const_1.color]: node.getAttribute(const_1.domColorAttr),
                        };
                    },
                },
            ],
            toDOM(node) {
                const colorValue = node.attrs[const_1.color];
                return [
                    'span',
                    {
                        class: [const_1.className, `${const_1.className}--${colorValue}`].join(' '),
                        [const_1.domColorAttr]: colorValue,
                    },
                    0,
                ];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: const_1.color,
                type: 'mark',
                getAttrs(token) {
                    return {
                        [const_1.color]: token.info,
                    };
                },
            },
        },
        toYfm: {
            open: (_state, mark) => {
                return `{${mark.attrs[const_1.color]}}(`;
            },
            close: ')',
            mixable: true,
            expelEnclosingWhitespace: true,
        },
    }))
        .addAction(const_1.colorAction, ({ schema }) => {
        const type = colorType(schema);
        return {
            isActive: (state) => Boolean((0, marks_1.isMarkActive)(state, type)),
            isEnable: (0, prosemirror_commands_1.toggleMark)(type),
            run: (state, dispatch, _view, attrs) => {
                const params = attrs;
                const hasMark = (0, marks_1.isMarkActive)(state, type);
                if (!params || !params[const_1.color]) {
                    if (!hasMark)
                        return true;
                    // remove mark
                    return (0, prosemirror_commands_1.toggleMark)(type, params)(state, dispatch);
                }
                if (hasMark) {
                    // remove old mark, then add new with new color
                    return (0, utils_1.chainAND)((0, prosemirror_commands_1.toggleMark)(type), (0, prosemirror_commands_1.toggleMark)(type, params))(state, dispatch);
                }
                // add mark
                return (0, prosemirror_commands_1.toggleMark)(type, params)(state, dispatch);
            },
            meta(state) {
                var _a;
                return (_a = type.isInSet(state.selection.$to.marks())) === null || _a === void 0 ? void 0 : _a.attrs[const_1.color];
            },
        };
    });
};
exports.Color = Color;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.ColorE = (0, core_1.createExtension)((b) => b.use(exports.Color));
