"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exitFromCutTitle = exports.liftEmptyBlockFromCut = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const schema_1 = require("../../../utils/schema");
const selection_1 = require("../../../utils/selection");
const nodes_1 = require("../../../utils/nodes");
const const_1 = require("./const");
const liftEmptyBlockFromCut = (state, dispatch) => {
    const { selection, schema } = state;
    if (!(0, selection_1.isTextSelection)(selection))
        return false;
    const { $cursor } = selection;
    // cursor should be inside an empty textblock
    if (!$cursor || $cursor.parent.content.size)
        return false;
    if ($cursor.depth > 2 && // yfm_cut -> yfm_cut_content -> <textblock>, depth must be at least 3
        (0, schema_1.isSameNodeType)($cursor.node(-1), (0, const_1.cutContentType)(schema)) &&
        (0, schema_1.isSameNodeType)($cursor.node(-2), (0, const_1.cutType)(schema))) {
        // current texblock is last child of yfm_cut_content
        if ($cursor.after() === $cursor.end(-1)) {
            if (dispatch) {
                dispatch(state.tr.lift($cursor.blockRange(), $cursor.depth - 3).scrollIntoView());
            }
            return true;
        }
    }
    return false;
};
exports.liftEmptyBlockFromCut = liftEmptyBlockFromCut;
const exitFromCutTitle = (state, dispatch) => {
    const { selection, schema } = state;
    if (!(0, selection_1.isTextSelection)(selection))
        return false;
    const { $cursor } = selection;
    if (!$cursor)
        return false;
    if (!(0, schema_1.isSameNodeType)($cursor.parent, (0, const_1.cutTitleType)(schema)) ||
        !(0, schema_1.isSameNodeType)($cursor.node(-1), (0, const_1.cutType)(schema))) {
        return false;
    }
    const cutNode = $cursor.node(-1);
    const cutContentFirstTextblockChild = (0, nodes_1.findFirstTextblockChild)(cutNode.lastChild);
    if (!cutContentFirstTextblockChild)
        return false;
    const cutContentPos = $cursor.start(-1) + cutNode.firstChild.nodeSize + 1;
    const targetPos = cutContentPos + cutContentFirstTextblockChild.offset;
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.setSelection(prosemirror_state_1.TextSelection.create(state.doc, targetPos)));
    return true;
};
exports.exitFromCutTitle = exitFromCutTitle;
