"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmCutE = exports.YfmCut = exports.cutContentType = exports.cutTitleType = exports.cutType = exports.CutNode = void 0;
const tslib_1 = require("tslib");
const log_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/log"));
const cut_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/plugins/cut"));
const prosemirror_commands_1 = require("prosemirror-commands");
const core_1 = require("../../../core");
const inputrules_1 = require("../../../utils/inputrules");
const toYfm_1 = require("./toYfm");
const const_1 = require("./const");
const fromYfm_1 = require("./fromYfm");
const spec_1 = require("./spec");
const toYfmCut_1 = require("./actions/toYfmCut");
const commands_1 = require("./commands");
const cutAction = 'toYfmCut';
var const_2 = require("./const");
Object.defineProperty(exports, "CutNode", { enumerable: true, get: function () { return const_2.CutNode; } });
Object.defineProperty(exports, "cutType", { enumerable: true, get: function () { return const_2.cutType; } });
Object.defineProperty(exports, "cutTitleType", { enumerable: true, get: function () { return const_2.cutTitleType; } });
Object.defineProperty(exports, "cutContentType", { enumerable: true, get: function () { return const_2.cutContentType; } });
const YfmCut = (builder, opts) => {
    const spec = (0, spec_1.getSpec)(opts);
    builder
        .configureMd((md) => md.use(cut_1.default, { log: log_1.default }))
        .addNode(const_1.CutNode.Cut, () => ({
        spec: spec[const_1.CutNode.Cut],
        toYfm: toYfm_1.toYfm[const_1.CutNode.Cut],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.CutNode.Cut],
        },
        // FIX: ignore mutation and don't rerender node when yfm.js open or close cut
        // @ts-expect-error
        view: () => () => ({
            ignoreMutation(mutation) {
                return mutation instanceof MutationRecord && mutation.type === 'attributes';
            },
        }),
    }))
        .addNode(const_1.CutNode.CutTitle, () => ({
        spec: spec[const_1.CutNode.CutTitle],
        toYfm: toYfm_1.toYfm[const_1.CutNode.CutTitle],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.CutNode.CutTitle],
        },
    }))
        .addNode(const_1.CutNode.CutContent, () => ({
        spec: spec[const_1.CutNode.CutContent],
        toYfm: toYfm_1.toYfm[const_1.CutNode.CutContent],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.CutNode.CutContent],
        },
    }))
        .addAction(cutAction, () => toYfmCut_1.toYfmCut)
        .addKeymap(() => ({
        Enter: (0, prosemirror_commands_1.chainCommands)(commands_1.exitFromCutTitle, commands_1.liftEmptyBlockFromCut),
    }))
        .addInputRules(({ schema }) => ({
        rules: [(0, inputrules_1.nodeInputRule)(/(?:^)({% cut)\s$/, (0, const_1.cutType)(schema).createAndFill(), 1)],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.yfmCutKey) {
        const { yfmCutKey } = opts;
        builder.addKeymap(() => ({ [yfmCutKey]: toYfmCut_1.createYfmCut }));
    }
};
exports.YfmCut = YfmCut;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.YfmCutE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.YfmCut, o));
