"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmHeadingE = exports.YfmHeading = exports.YfmHeadingAttr = void 0;
const core_1 = require("../../../core");
const const_1 = require("./const");
const utils_1 = require("./utils");
const actions_1 = require("./actions");
const commands_1 = require("./commands");
const DEFAULT_PLACEHOLDER = (node) => 'Heading ' + node.attrs[const_1.YfmHeadingAttr.Level];
var const_2 = require("./const");
Object.defineProperty(exports, "YfmHeadingAttr", { enumerable: true, get: function () { return const_2.YfmHeadingAttr; } });
/** YfmHeading extension needs markdown-it-attrs plugin */
const YfmHeading = (builder, opts) => {
    const { headingPlaceholder } = opts !== null && opts !== void 0 ? opts : {};
    builder.addNode(const_1.heading, () => ({
        spec: {
            attrs: {
                [const_1.YfmHeadingAttr.Id]: { default: '' },
                [const_1.YfmHeadingAttr.Level]: { default: 1 },
            },
            content: '(text | inline)*',
            group: 'block',
            defining: true,
            selectable: false,
            parseDOM: [
                { tag: 'h1', getAttrs: (0, utils_1.getNodeAttrs)(1) },
                { tag: 'h2', getAttrs: (0, utils_1.getNodeAttrs)(2) },
                { tag: 'h3', getAttrs: (0, utils_1.getNodeAttrs)(3) },
                { tag: 'h4', getAttrs: (0, utils_1.getNodeAttrs)(4) },
                { tag: 'h5', getAttrs: (0, utils_1.getNodeAttrs)(5) },
                { tag: 'h6', getAttrs: (0, utils_1.getNodeAttrs)(6) },
            ],
            toDOM(node) {
                const id = node.attrs[const_1.YfmHeadingAttr.Id];
                return [
                    'h' + node.attrs[const_1.YfmHeadingAttr.Level],
                    id ? { id } : {},
                    0,
                    // [
                    //     'a',
                    //     {
                    //         href: `#${node.attrs[YfmHeadingAttr.Id]}`,
                    //         class: 'yfm-anchor',
                    //         'aria-hidden': 'true',
                    //         contenteditable: 'false',
                    //     },
                    // ],
                    // ['span', 0],
                ];
            },
            placeholder: {
                content: headingPlaceholder !== null && headingPlaceholder !== void 0 ? headingPlaceholder : DEFAULT_PLACEHOLDER,
                alwaysVisible: true,
            },
        },
        fromYfm: {
            tokenSpec: {
                name: const_1.heading,
                type: 'block',
                getAttrs: (token) => {
                    if (token.type.endsWith('_close'))
                        return {};
                    const attrs = Object.fromEntries(token.attrs || []);
                    // if (!attrs[YfmHeadingAttr.Id]) {
                    //     // calculate id if it was not specified
                    //     // tokens[index + 1] is child inline token
                    //     attrs[YfmHeadingAttr.Id] = slugify(tokens[index + 1].content);
                    // }
                    // attrs have id only if it explicitly specified manually
                    return Object.assign({ [const_1.YfmHeadingAttr.Level]: Number(token.tag.slice(1)) }, attrs);
                },
            },
        },
        toYfm: (state, node) => {
            state.write(state.repeat('#', node.attrs[const_1.YfmHeadingAttr.Level]) + ' ');
            state.renderInline(node);
            const anchor = node.attrs[const_1.YfmHeadingAttr.Id];
            if (anchor /*&& anchor !== node.firstChild?.textContent*/) {
                state.write(` {#${anchor}}`);
            }
            state.closeBlock(node);
        },
    }));
    builder
        .addKeymap(() => {
        const { h1Key, h2Key, h3Key, h4Key, h5Key, h6Key } = opts !== null && opts !== void 0 ? opts : {};
        const bindings = { Backspace: commands_1.resetHeading };
        if (h1Key)
            bindings[h1Key] = (0, commands_1.toHeading)(1);
        if (h2Key)
            bindings[h2Key] = (0, commands_1.toHeading)(2);
        if (h3Key)
            bindings[h3Key] = (0, commands_1.toHeading)(3);
        if (h4Key)
            bindings[h4Key] = (0, commands_1.toHeading)(4);
        if (h5Key)
            bindings[h5Key] = (0, commands_1.toHeading)(5);
        if (h6Key)
            bindings[h6Key] = (0, commands_1.toHeading)(6);
        return bindings;
    })
        .addInputRules(({ schema }) => ({ rules: [(0, utils_1.headingRule)((0, utils_1.hType)(schema), 6)] }));
    builder
        .addAction(const_1.HeadingAction.ToH1, () => (0, actions_1.headingAction)(1))
        .addAction(const_1.HeadingAction.ToH2, () => (0, actions_1.headingAction)(2))
        .addAction(const_1.HeadingAction.ToH3, () => (0, actions_1.headingAction)(3))
        .addAction(const_1.HeadingAction.ToH4, () => (0, actions_1.headingAction)(4))
        .addAction(const_1.HeadingAction.ToH5, () => (0, actions_1.headingAction)(5))
        .addAction(const_1.HeadingAction.ToH6, () => (0, actions_1.headingAction)(6));
};
exports.YfmHeading = YfmHeading;
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
exports.YfmHeadingE = (0, core_1.createExtension)((b, o = {}) => b.use(exports.YfmHeading, o));
