"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exitFromNoteTitle = void 0;
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const schema_1 = require("../../../utils/schema");
const selection_1 = require("../../../utils/selection");
const nodes_1 = require("../../../utils/nodes");
const utils_1 = require("./utils");
const exitFromNoteTitle = (state, dispatch) => {
    const { selection, schema } = state;
    if (!(0, selection_1.isTextSelection)(selection))
        return false;
    const { $cursor } = selection;
    if (!$cursor)
        return false;
    if (!(0, schema_1.isSameNodeType)($cursor.parent, (0, utils_1.noteTitleType)(schema)) ||
        !(0, schema_1.isSameNodeType)($cursor.node(-1), (0, utils_1.noteType)(schema))) {
        return false;
    }
    const noteNode = $cursor.node(-1);
    const noteContentFirstTextblockChild = (0, nodes_1.findFirstTextblockChild)(getNoteContent(noteNode));
    if (!noteContentFirstTextblockChild)
        return false;
    const noteContentPos = $cursor.start(-1) + noteNode.firstChild.nodeSize;
    const targetPos = noteContentPos + noteContentFirstTextblockChild.offset;
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.setSelection(prosemirror_state_1.TextSelection.create(state.doc, targetPos)));
    return true;
};
exports.exitFromNoteTitle = exitFromNoteTitle;
function getNoteContent(noteNode) {
    const noteChildren = [];
    noteNode.forEach((node) => noteChildren.push(node));
    // first child node is note title
    return prosemirror_model_1.Fragment.from(noteChildren.slice(1));
}
