"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.appendColumn = void 0;
const utils_1 = require("../utils");
const helpers_1 = require("../helpers");
const prosemirror_utils_1 = require("prosemirror-utils");
const __1 = require("..");
const appendColumn = (state, dispatch, _, attrs) => {
    var _a;
    if (!attrs)
        return false;
    const { tablePos, columnNumber, direction } = attrs;
    const parentTable = (_a = (0, prosemirror_utils_1.findParentNodeClosestToPos)(state.doc.resolve(tablePos + 1), __1.isTableNode)) === null || _a === void 0 ? void 0 : _a.node;
    if (!parentTable)
        return false;
    let parentCell;
    let parentRow;
    if (columnNumber !== undefined) {
        parentCell = (0, utils_1.findChildTableCells)(parentTable)[columnNumber];
        parentRow = (0, prosemirror_utils_1.findParentNodeClosestToPos)(state.doc.resolve(tablePos + parentCell.pos + 1), __1.isTableRowNode);
    }
    else {
        parentCell = (0, utils_1.findChildTableCells)(parentTable).pop();
        parentRow = (0, utils_1.findChildTableRows)(parentTable).pop();
    }
    if (!parentCell || !parentRow || !parentTable) {
        return false;
    }
    const parentCellIndex = columnNumber || (0, helpers_1.findChildIndex)(parentRow.node, parentCell.node);
    if (parentCellIndex < 0) {
        return false;
    }
    if (dispatch) {
        const allRows = (0, utils_1.findChildTableRows)(parentTable);
        let tr = state.tr;
        for (const row of allRows) {
            const rowCells = (0, utils_1.findChildTableCells)(row.node);
            const cell = rowCells[parentCellIndex];
            let position = tablePos + row.pos + cell.pos + 2;
            position += direction === 'before' ? 0 : cell.node.nodeSize;
            tr = tr.insert(tr.mapping.map(position), cell.node.type.createAndFill(cell.node.attrs));
        }
        dispatch(tr.scrollIntoView());
    }
    return true;
};
exports.appendColumn = appendColumn;
