/// <reference types="react" />
import type { IconProps } from '@yandex-cloud/uikit';
import type { ClassNameProps } from '../classname';
import type { ToolbarListButtonData } from './ToolbarListButton';
export declare type ToolbarBaseProps<E> = ClassNameProps & {
    editor: E;
    focus(): void;
    onClick?(id: string, attrs?: {
        [key: string]: any;
    }): void;
};
export declare type ToolbarIconData = Pick<IconProps, 'data' | 'size'>;
export declare type ToolbarItemData<E> = {
    id: string;
    icon: ToolbarIconData;
    title: string | (() => string);
    hotkey?: string | null;
    exec(editor: E): void;
    isActive(editor: E): boolean;
    isEnable(editor: E): boolean;
};
export declare enum ToolbarDataType {
    SingleButton = "s-button",
    ListButton = "list-b",
    ReactNode = "r-node",
    ReactNodeFn = "r-node-fn",
    ReactComponent = "r-component"
}
export declare type ToolbarGroupItemData<E> = ToolbarSingleItemData<E> | ToolbarListItemData<E> | ToolbarReactNodeData | ToolbarReactNodeFnData<E> | ToolbarReactComponentData<E>;
export declare type ToolbarSingleItemData<E> = ToolbarItemData<E> & {
    id: string;
    type: ToolbarDataType.SingleButton;
    className?: string;
};
export declare type ToolbarListItemData<E> = ToolbarListButtonData<E> & {
    id: string;
    type: ToolbarDataType.ListButton;
    className?: string;
};
export declare type ToolbarReactComponentData<E> = {
    id: string;
    type: ToolbarDataType.ReactComponent;
    width: number;
    className?: string;
    component: React.ComponentType<ToolbarBaseProps<E>>;
};
export declare type ToolbarReactNodeData = {
    id: string;
    type: ToolbarDataType.ReactNode;
    width: number;
    content: React.ReactNode;
};
export declare type ToolbarReactNodeFnData<E> = {
    id: string;
    type: ToolbarDataType.ReactNodeFn;
    width: number;
    content: (e: E) => React.ReactNode;
};
