"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isCodeBlock = exports.isSelectableNode = exports.isNodeEmpty = exports.findFirstTextblockChild = void 0;
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_utils_1 = require("prosemirror-utils");
function findFirstTextblockChild(parent) {
    if (parent instanceof prosemirror_model_1.Node && (!parent.isBlock || parent.isAtom))
        return null;
    const children = [];
    parent.forEach((node, offset) => children.push({ node, offset }));
    for (const child of children) {
        if (child.node.isTextblock)
            return child;
        const nestedTextBlockChild = findFirstTextblockChild(child.node);
        if (nestedTextBlockChild) {
            return {
                node: nestedTextBlockChild.node,
                offset: child.offset + nestedTextBlockChild.offset,
            };
        }
    }
    return null;
}
exports.findFirstTextblockChild = findFirstTextblockChild;
const isNodeEmpty = (node) => {
    const emptyChildren = (0, prosemirror_utils_1.findChildren)(node, (n) => (!n.isText && !n.isAtom && n.content.size === 0) || (n.isText && !n.textContent), true);
    let descendantsCount = 0;
    node.descendants(() => {
        descendantsCount++;
    });
    return emptyChildren.length === descendantsCount;
};
exports.isNodeEmpty = isNodeEmpty;
const isSelectableNode = (node) => {
    if (node.type.spec.selectable === false)
        return false;
    if (node.type.isText && node.type.spec.selectable !== true)
        return false;
    return true;
};
exports.isSelectableNode = isSelectableNode;
function isCodeBlock(node) {
    return node.isTextblock && Boolean(node.type.spec.code);
}
exports.isCodeBlock = isCodeBlock;
