import { DOMSerializer } from 'prosemirror-model';
import { Decoration, DecorationSet } from 'prosemirror-view';
import { NodeSelection, Plugin, PluginKey } from 'prosemirror-state';
import { pType } from '../../base/BaseSchema';
import { createPlaceholder } from '../../behavior/Placeholder';
import { GapCursorSelection } from './GapCursorSelection';
import './gapcursor.css';
const key = new PluginKey('gapCursorPlugin');
export const gapCursor = () => new Plugin({
    key,
    state: {
        init: () => false,
        apply: (_tr, _pluginState, _oldState, newState) => {
            return (newState.selection instanceof GapCursorSelection ||
                newState.selection instanceof NodeSelection);
        },
    },
    view() {
        return {
            update(view) {
                const pluginState = key.getState(view.state);
                view.dom.classList.toggle('Prosemirror-hide-cursor', pluginState);
            },
        };
    },
    props: {
        decorations: ({ doc, selection }) => {
            if (selection instanceof GapCursorSelection) {
                const position = selection.head;
                return DecorationSet.create(doc, [
                    Decoration.widget(position, toDOM, {
                        key: 'gapcursor',
                        side: -1,
                    }),
                ]);
            }
            return null;
        },
    },
});
const toDOM = (view, getPos) => {
    const node = pType(view.state.schema).create();
    const element = DOMSerializer.fromSchema(view.state.schema).serializeNode(node);
    element.appendChild(createPlaceholder(node, true));
    element.classList.add('ye-gapcursor');
    element.addEventListener('mousedown', () => {
        const pos = getPos();
        if (pos !== undefined) {
            view.dispatch(view.state.tr.replaceSelectionWith(node));
        }
    });
    return element;
};
