import { Plugin, PluginKey } from 'prosemirror-state';
import { Decoration, DecorationSet } from 'prosemirror-view';
import { findChildren, findParentNodeClosestToPos } from 'prosemirror-utils';
import { cn } from '../../../classname';
import { isNodeEmpty } from '../../../utils/nodes';
import { isTextSelection } from '../../../utils/selection';
import './index.css';
export const getPlaceholderContent = (node) => {
    var _a;
    const content = ((_a = node.type.spec.placeholder) === null || _a === void 0 ? void 0 : _a.content) || '';
    return typeof content === 'function' ? content(node) : content;
};
const getPlaceholderPluginKeys = (schema) => {
    var _a;
    const pluginKeys = [];
    for (const node in schema.nodes) {
        if (schema.nodes[node]) {
            const spec = schema.nodes[node].spec;
            if ((_a = spec.placeholder) === null || _a === void 0 ? void 0 : _a.customPlugin) {
                pluginKeys.push(spec.placeholder.customPlugin);
            }
        }
    }
    return pluginKeys;
};
const b = cn('placeholder');
export const createPlaceholder = (node, focus) => {
    const placeholder = document.createElement('div');
    placeholder.classList.add(...b({ focus }).split(' '));
    placeholder.textContent = getPlaceholderContent(node);
    return placeholder;
};
const placeholderNeeded = (node) => {
    const childrenWithPlaceholderVisible = findChildren(node, (n) => { var _a; return Boolean((_a = n.type.spec.placeholder) === null || _a === void 0 ? void 0 : _a.alwaysVisible); });
    return (isNodeEmpty(node) &&
        // Если есть дочерние ноды у которых есть постоянный плейсхолдер - отдаем им приоритет
        !childrenWithPlaceholderVisible.length);
};
const addDecoration = (decorationsMap, node, pos, cursorPos) => {
    const placeholderSpec = node.type.spec.placeholder;
    const decorationPosition = pos + node.childCount + 1;
    // Не добавляем декорацию если на этой позиции уже есть плейсхолдер
    if (!placeholderSpec || decorationsMap[decorationPosition])
        return;
    if (placeholderSpec.customPlugin) {
        decorationsMap[decorationPosition] = placeholderSpec.customPlugin;
        return;
    }
    decorationsMap[decorationPosition] = Decoration.widget(decorationPosition, createPlaceholder(node, decorationPosition === cursorPos));
};
export const Placeholder = (builder) => {
    builder.addPlugin(() => new Plugin({
        props: {
            decorations: (state) => {
                var _a;
                const decorationsMap = {};
                const { selection } = state;
                const cursorPos = isTextSelection(selection)
                    ? (_a = selection.$cursor) === null || _a === void 0 ? void 0 : _a.pos
                    : null;
                getPlaceholderPluginKeys(state.schema).forEach((f) => {
                    var _a;
                    // Используем find потому что при помощи него можно проитерировать по DecorationSet.
                    (_a = f.getState(state)) === null || _a === void 0 ? void 0 : _a.find(undefined, undefined, (spec) => {
                        decorationsMap[spec.pos] = f;
                        return false;
                    });
                });
                // Рисуем плэйсхолдеры для всех нод у которых плэйсхолдер alwaysVisible
                const decorate = (node, pos) => {
                    const placeholderSpec = node.type.spec.placeholder;
                    if (placeholderSpec &&
                        placeholderSpec.alwaysVisible &&
                        placeholderNeeded(node)) {
                        addDecoration(decorationsMap, node, pos, cursorPos);
                    }
                };
                state.doc.descendants(decorate);
                const parentNode = findParentNodeClosestToPos(state.selection.$from, (node) => {
                    return Boolean(node.type.spec.placeholder);
                });
                const placeholderSpec = parentNode === null || parentNode === void 0 ? void 0 : parentNode.node.type.spec.placeholder;
                // Дорисовываем плэйсхолдер, если его нужно отрисовать на месте курсора
                if (parentNode &&
                    placeholderNeeded(parentNode.node) &&
                    placeholderSpec &&
                    !placeholderSpec.alwaysVisible) {
                    addDecoration(decorationsMap, parentNode.node, parentNode.pos, cursorPos);
                }
                const decorations = Object.values(decorationsMap).filter((decoration) => !(decoration instanceof PluginKey));
                return DecorationSet.create(state.doc, decorations);
            },
        },
    }));
};
