import { lift, wrapIn } from 'prosemirror-commands';
import { NodeSelection } from 'prosemirror-state';
import { isTextSelection } from '../../../utils/selection';
import { bqType } from './const';
import '../../../types/spec';
function getCursor(sel) {
    if (isTextSelection(sel))
        return sel.$cursor;
    return undefined;
}
export const liftFromQuote = (state, dispatch) => {
    const $cursor = getCursor(state.selection);
    if (!$cursor || $cursor.parentOffset > 0 || $cursor.node(-1).type !== bqType(state.schema)) {
        return false;
    }
    return lift(state, dispatch);
};
export const selectQuoteBeforeCursor = (state, dispatch) => {
    const $cursor = getCursor(state.selection);
    if (!$cursor || $cursor.parentOffset > 0)
        return false;
    const index = $cursor.index(-1);
    const nodeBefore = $cursor.node(-1).maybeChild(index - 1);
    if (!nodeBefore || nodeBefore.type !== bqType(state.schema))
        return false;
    const beforePos = $cursor.before();
    dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.setSelection(NodeSelection.create(state.doc, beforePos - nodeBefore.nodeSize)));
    return true;
};
export const toggleQuote = (state, dispatch) => {
    const { selection } = state;
    const qType = bqType(state.schema);
    if (!isTextSelection(selection) || !selection.$cursor)
        return wrapIn(qType)(state, dispatch);
    const { $cursor } = selection;
    let { depth } = $cursor;
    while (depth >= 0) {
        const node = $cursor.node(depth);
        const nodeSpec = node.type.spec;
        if (!nodeSpec.complex || nodeSpec.complex === 'root') {
            const targetDepth = depth - 1;
            const range = getBlockRange($cursor, depth);
            if ($cursor.node(targetDepth).type === qType) {
                dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.lift(range, targetDepth - 1).scrollIntoView());
            }
            else {
                dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.wrap(range, [{ type: qType }]).scrollIntoView());
            }
            return true;
        }
        depth--;
    }
    return false;
};
function getBlockRange($pos, depth) {
    const { doc } = $pos;
    const $before = doc.resolve($pos.before(depth));
    const $after = doc.resolve($pos.after(depth));
    return $before.blockRange($after);
}
