import { chainCommands, wrapIn } from 'prosemirror-commands';
import { wrappingInputRule } from 'prosemirror-inputrules';
import { hasParentNodeOfType } from 'prosemirror-utils';
import { createExtension } from '../../../core';
import { selectQuoteBeforeCursor, liftFromQuote, toggleQuote } from './commands';
import { blockquote, bqType } from './const';
export { blockquote };
const bqAction = 'quote';
export const Blockquote = (builder, opts) => {
    builder.addNode(blockquote, () => ({
        spec: {
            content: 'block+',
            group: 'block',
            defining: true,
            parseDOM: [{ tag: 'blockquote' }],
            toDOM() {
                return ['blockquote', 0];
            },
        },
        fromYfm: { tokenSpec: { name: blockquote, type: 'block' } },
        toYfm: (state, node) => {
            state.wrapBlock('> ', null, node, () => state.renderContent(node));
        },
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.qouteKey) {
        const { qouteKey } = opts;
        builder.addKeymap(({ schema }) => ({ [qouteKey]: wrapIn(bqType(schema)) }));
    }
    builder.addKeymap(() => ({
        Backspace: chainCommands(liftFromQuote, selectQuoteBeforeCursor),
    }));
    builder.addInputRules(({ schema }) => ({ rules: [blockQuoteRule(bqType(schema))] }));
    builder.addAction(bqAction, ({ schema }) => {
        const bq = bqType(schema);
        return {
            isActive: (state) => hasParentNodeOfType(bq)(state.selection),
            isEnable: toggleQuote,
            run: toggleQuote,
        };
    });
};
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
export const BlockquoteE = createExtension((b, o = {}) => b.use(Blockquote, o));
/**
 * Given a blockquote node type, returns an input rule that turns `"> "`
 * at the start of a textblock into a blockquote.
 */
function blockQuoteRule(nodeType) {
    return wrappingInputRule(/^\s*>\s$/, nodeType);
}
