import { liftListItem, sinkListItem, splitListItem } from 'prosemirror-schema-list';
import { createExtension } from '../../../core';
import { actions } from './actions';
import { ListAction, ListNode } from './const';
import { fromYfm } from './fromYfm';
import { spec } from './spec';
import { toYfm } from './toYfm';
import { ListsInputRulesExtension } from './inputrules';
import { blType, liType, olType } from './utils';
import { toList } from './commands';
export const Lists = (builder, opts) => {
    builder
        .addNode(ListNode.ListItem, () => ({
        spec: spec[ListNode.ListItem],
        toYfm: toYfm[ListNode.ListItem],
        fromYfm: { tokenSpec: fromYfm[ListNode.ListItem] },
    }))
        .addNode(ListNode.BulletList, () => ({
        spec: spec[ListNode.BulletList],
        toYfm: toYfm[ListNode.BulletList],
        fromYfm: { tokenSpec: fromYfm[ListNode.BulletList] },
    }))
        .addNode(ListNode.OrderedList, () => ({
        spec: spec[ListNode.OrderedList],
        toYfm: toYfm[ListNode.OrderedList],
        fromYfm: { tokenSpec: fromYfm[ListNode.OrderedList] },
    }));
    builder.addKeymap(({ schema }) => {
        const { ulKey, olKey } = opts !== null && opts !== void 0 ? opts : {};
        const bindings = {};
        if (ulKey)
            bindings[ulKey] = toList(blType(schema));
        if (olKey)
            bindings[olKey] = toList(olType(schema));
        return Object.assign({ Enter: splitListItem(liType(schema)), Tab: sinkListItem(liType(schema)), 'Shift-Tab': liftListItem(liType(schema)), 'Mod-[': liftListItem(liType(schema)), 'Mod-]': sinkListItem(liType(schema)) }, bindings);
    });
    builder.use(ListsInputRulesExtension, { bulletListInputRule: opts === null || opts === void 0 ? void 0 : opts.ulInputRules });
    builder
        .addAction(ListAction.ToBulletList, actions.toBulletList)
        .addAction(ListAction.ToOrderedList, actions.toOrderedList)
        .addAction(ListAction.SinkListItem, actions.sinkListItem)
        .addAction(ListAction.LiftListItem, actions.liftListItem);
};
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
export const ListsE = createExtension((b, o = {}) => b.use(Lists, o));
