import { toggleMark } from 'prosemirror-commands';
import { createExtension } from '../../../core';
import { createToggleMarkAction } from '../../../utils/actions';
import { markTypeFactory } from '../../../utils/schema';
import { markInputRule } from '../../../utils/inputrules';
export const strike = 'strike';
const sAction = 'strike';
const sType = markTypeFactory(strike);
export const Strike = (builder, opts) => {
    builder
        .addMark(strike, () => ({
        spec: {
            parseDOM: [{ tag: 'strike' }],
            toDOM() {
                return ['strike'];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: strike,
                type: 'mark',
            },
            tokenName: 's',
        },
        toYfm: { open: '~~', close: '~~', mixable: true, expelEnclosingWhitespace: true },
    }))
        .addAction(sAction, ({ schema }) => createToggleMarkAction(sType(schema)))
        .addInputRules(({ schema }) => ({
        rules: [markInputRule({ open: '~~', close: '~~', ignoreBetween: '~' }, sType(schema))],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.strikeKey) {
        const { strikeKey } = opts;
        builder.addKeymap(({ schema }) => ({ [strikeKey]: toggleMark(sType(schema)) }));
    }
};
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
export const StrikeE = createExtension((b, o = {}) => b.use(Strike, o));
