import { toggleMark } from 'prosemirror-commands';
import { createExtension } from '../../../core';
import { createToggleMarkAction } from '../../../utils/actions';
import { markTypeFactory } from '../../../utils/schema';
import { markInputRule } from '../../../utils/inputrules';
const ins = require('markdown-it-ins');
export const underline = 'ins';
const undAction = 'underline';
const undType = markTypeFactory(underline);
export const Underline = (builder, opts) => {
    builder
        .configureMd((md) => md.use(ins))
        .addMark(underline, () => ({
        spec: {
            parseDOM: [{ tag: 'ins' }],
            toDOM() {
                return ['ins'];
            },
        },
        toYfm: { open: '++', close: '++', mixable: true, expelEnclosingWhitespace: true },
        fromYfm: { tokenSpec: { name: underline, type: 'mark' } },
    }))
        .addAction(undAction, ({ schema }) => createToggleMarkAction(undType(schema)))
        .addInputRules(({ schema }) => ({
        rules: [markInputRule({ open: '++', close: '++', ignoreBetween: '+' }, undType(schema))],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.underlineKey) {
        const { underlineKey } = opts;
        builder.addKeymap(({ schema }) => ({ [underlineKey]: toggleMark(undType(schema)) }));
    }
};
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
export const UnderlineE = createExtension((b, o = {}) => b.use(Underline, o));
