import isNumber from 'is-number';
import imsize from '@doc-tools/transform/lib/plugins/imsize';
import { ImsizeAttr } from '@doc-tools/transform/lib/plugins/imsize/const';
import log from '@doc-tools/transform/lib/log';
import { createExtension } from '../../../core';
import { addImageAction, image } from './const';
import { addImage } from './actions';
export { ImageAttr } from './const';
export const ImgSize = (builder, opts) => {
    builder.configureMd((md) => md.use(imsize, { log }));
    builder.addNode(image, () => ({
        spec: {
            inline: true,
            attrs: {
                [ImsizeAttr.Src]: {},
                [ImsizeAttr.Alt]: { default: null },
                [ImsizeAttr.Title]: { default: null },
                [ImsizeAttr.Height]: { default: null },
                [ImsizeAttr.Width]: { default: null },
            },
            placeholder: opts.placeholder,
            group: 'inline',
            draggable: true,
            parseDOM: [
                {
                    tag: 'img[src]',
                    getAttrs(dom) {
                        const height = dom.getAttribute(ImsizeAttr.Height);
                        const width = dom.getAttribute(ImsizeAttr.Width);
                        return {
                            [ImsizeAttr.Src]: dom.getAttribute(ImsizeAttr.Src),
                            [ImsizeAttr.Alt]: dom.getAttribute(ImsizeAttr.Alt),
                            [ImsizeAttr.Title]: dom.getAttribute(ImsizeAttr.Title),
                            [ImsizeAttr.Height]: isNumber(height) ? height : null,
                            [ImsizeAttr.Width]: isNumber(width) ? height : null,
                        };
                    },
                },
            ],
            toDOM(node) {
                return ['img', node.attrs];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: image,
                type: 'node',
                getAttrs: (tok) => {
                    var _a, _b;
                    return ({
                        [ImsizeAttr.Src]: tok.attrGet(ImsizeAttr.Src),
                        [ImsizeAttr.Title]: tok.attrGet(ImsizeAttr.Title),
                        [ImsizeAttr.Height]: tok.attrGet(ImsizeAttr.Height),
                        [ImsizeAttr.Width]: tok.attrGet(ImsizeAttr.Width),
                        [ImsizeAttr.Alt]: ((_b = (_a = tok.children) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.content) || null,
                    });
                },
            },
        },
        toYfm: (state, node) => {
            const attrs = node.attrs;
            state.write('![' +
                state.esc(attrs.alt || '') +
                '](' +
                state.esc(attrs.src) +
                (attrs.title ? ' ' + state.quote(attrs.title) : '') +
                getSize(attrs) +
                ')');
        },
    }));
    builder.addAction(addImageAction, ({ schema }) => addImage(schema));
};
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
export const ImgSizeE = createExtension((b, o = {}) => b.use(ImgSize, o));
function getSize({ width, height }) {
    if (width || height) {
        return ` =${width || ''}x${height || ''}`;
    }
    return '';
}
