import log from '@doc-tools/transform/lib/log';
import yfmPlugin from '@doc-tools/transform/lib/plugins/notes';
import { createExtension } from '../../../core';
import { toYfm } from './toYfm';
import { NoteNode } from './const';
import { fromYfm } from './fromYfm';
import { getSpec } from './spec';
import { createYfmNote, toYfmNote } from './actions/toYfmNote';
import { nodeInputRule } from '../../../utils/inputrules';
import { exitFromNoteTitle } from './commands';
import { noteType } from './utils';
import './index.css';
const noteAction = 'toYfmNote';
export { noteType, noteTitleType } from './utils';
export const YfmNote = (builder, opts) => {
    const spec = getSpec(opts);
    builder
        .configureMd((md) => md.use(yfmPlugin, { log }))
        .addNode(NoteNode.Note, () => ({
        spec: spec[NoteNode.Note],
        toYfm: toYfm[NoteNode.Note],
        fromYfm: {
            tokenSpec: fromYfm[NoteNode.Note],
        },
    }))
        .addNode(NoteNode.NoteTitle, () => ({
        spec: spec[NoteNode.NoteTitle],
        toYfm: toYfm[NoteNode.NoteTitle],
        fromYfm: {
            tokenSpec: fromYfm[NoteNode.NoteTitle],
        },
    }))
        .addKeymap(() => ({ Enter: exitFromNoteTitle }))
        .addAction(noteAction, () => toYfmNote)
        .addInputRules(({ schema }) => ({
        rules: [nodeInputRule(/(?:^)({% note)\s$/, noteType(schema).createAndFill(), 1)],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.yfmNoteKey) {
        const { yfmNoteKey } = opts;
        builder.addKeymap(() => ({ [yfmNoteKey]: createYfmNote }));
    }
};
/**
 * @deprecated
 * For tests only.
 * Remove after WIKI-16660
 */
export const YfmNoteE = createExtension((b, o = {}) => b.use(YfmNote, o));
