import React from 'react';
import { isFunction } from 'lodash';
import { Button, Icon, Menu, Popup } from '@yandex-cloud/uikit';
import chevronIcon from '../../../assets/icons/ye-chevron.svg';
import { cn } from '../classname';
import { useBooleanState } from '../react-utils/hooks';
import './ToolbarListButton.css';
const b = cn('toolbar-list-button');
export function ToolbarListButton({ className, editor, focus, onClick, icon, withArrow, data, }) {
    var _a;
    const buttonRef = React.useRef(null);
    const [open, , hide, toggleOpen] = useBooleanState(false);
    const someActive = data.some((item) => item.isActive(editor));
    const everyDisabled = data.every((item) => !item.isEnable(editor));
    const popupOpen = everyDisabled ? false : open;
    const shouldForceHide = open && !popupOpen;
    React.useEffect(() => {
        if (shouldForceHide) {
            hide();
        }
    }, [hide, shouldForceHide]);
    if (data.length === 0)
        return null;
    const buttonContent = [React.createElement(Icon, { key: 1, data: icon.data, size: (_a = icon.size) !== null && _a !== void 0 ? _a : 16 })];
    if (withArrow) {
        buttonContent.push(React.createElement(React.Fragment, { key: 2 }, ''));
        buttonContent.push(React.createElement(Icon, { key: 3, data: chevronIcon, size: 16 }));
    }
    return (React.createElement(React.Fragment, null,
        React.createElement(Button, { size: "m", ref: buttonRef, view: someActive || popupOpen ? 'normal' : 'flat', selected: someActive, disabled: everyDisabled, className: b({ arrow: withArrow }, [className]), onClick: toggleOpen }, buttonContent),
        React.createElement(Popup, { anchorRef: buttonRef, open: popupOpen, onClose: hide },
            React.createElement(Menu, { size: "l", className: b('menu') }, data.map(({ id, title, icon, hotkey, isActive, isEnable, exec }) => {
                var _a;
                const titleText = isFunction(title) ? title() : title;
                return (React.createElement(Menu.Item, { key: id, active: isActive(editor), disabled: !isEnable(editor), onClick: () => {
                        hide();
                        focus();
                        exec(editor);
                        onClick === null || onClick === void 0 ? void 0 : onClick(id);
                    }, icon: React.createElement(Icon, { data: icon.data, size: (_a = icon.size) !== null && _a !== void 0 ? _a : 16 }), extraProps: { 'aria-label': titleText } },
                    React.createElement("div", { className: b('item') },
                        titleText,
                        hotkey && React.createElement("span", { className: b('hotkey') }, hotkey))));
            })))));
}
